{# Import the theme's layout. #}
{% extends "!layout.html" %}

{%- block extrahead %}
{# Enable versions switch #}
{% if enable_versions_switch %}
  <!-- Point to the *latest* version switcher. This will allow the latest versions to appear on older documentation. -->
  <script type="text/javascript" src="/latest/_static/version_switch.js"></script>
{% endif %}
{% endblock %}

{%- block scripts %}
    <script src="{{ pathto('_static/jquery.js', resource=True) }}"></script>
    <script src="{{ pathto('_static/underscore.js', resource=True) }}"></script>
    {{ super() }}
{%- endblock %}

{% block menu %}
    {{ super() }}

    {% if menu_links %}
        <p class="caption">
            <span class="caption-text">Getting help</span>
        </p>
        <ul>
            {% for text, link in menu_links %}
                <li class="toctree-l1"><a href="{{ link }}">{{ text }}</a></li>
            {% endfor %}
        </ul>
    {% endif %}
{% endblock %}
