#!/usr/bin/env bash
# Test the new +f modifier to -B
gmt begin fancyannot ps
  gmt subplot begin 2x1 -M12p -Fs6i/4i -Sr -R-1.5/1.5/-1/1 -T"FANCY ANNOTATIONS"
    gmt basemap -JM? -B+f 
    gmt basemap -JM? -c
  gmt subplot end
gmt end show
