# Symbols made for GMT by Jose A. Alvarez-Gomez
#
# Geological symbol: Plane Inverted
#
# The symbol reads the strike and dip in the 3rd and 4th columns in psxy
#
N: 2 ao
if $1 [> 0:90 then 0.5 -0.5 0.3 $2 l+fHelvetica-Oblique+jBR -W-
if $1 [> 90:180 then 0.5 0.5 0.3 $2 l+fHelvetica-Oblique+jTR -W-
if $1 [> 180:270 then -0.5 0.5 0.3 $2 l+fHelvetica-Oblique+jTL -W-
if $1 [> 270:360 then -0.5 -0.5 0.3 $2 l+fHelvetica-Oblique+jBL -W-
$1 R
0 0 1 y
-0.0625 0.25 0.126 -
-0.0625 0 0.126 -
-0.125 0.125 0.25 90 270 m
0.125 0 0.25 -
