/*--------------------------------------------------------------------
 *
 *  Copyright (c) 2013-2025 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
 *  See LICENSE.TXT file for copying and redistribution conditions.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; version 3 or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  Contact info: www.generic-mapping-tools.org
 *--------------------------------------------------------------------*/

/*!
 * \file seis_defaults.h
 * \brief
 */

#ifndef SEIS_DEFAULTS_H
#define SEIS_DEFAULTS_H

#define SEIS_MAG_REFERENCE 5.0		/* Reference magnitude for -S */
#define SEIS_MOMENT_MANT_REFERENCE 4.0	/* Mantissa for reference moment for -S */
#define SEIS_MOMENT_EXP_REFERENCE 23	/* Exponent for reference moment for -S */

/* Default font, offset, and symbol sizes */
#define SEIS_DEFAULT_FONTSIZE		9.0	/* In points */
#define SEIS_DEFAULT_OFFSET		3.0	/* In points */
#define SEIS_DEFAULT_SYMBOL_SIZE	6.0	/* In points */

#endif /* SEIS_DEFAULTS_H */
