		{"Adindan", "Clarke-1880", "Burkina Faso", {-118, -14, 218}},
		{"Adindan", "Clarke-1880", "Cameroon", {-134, -2, 210}},
		{"Adindan", "Clarke-1880", "Ethiopia", {-165, -11, 206}},
		{"Adindan", "Clarke-1880", "Mali", {-123, -20, 220}},
		{"Adindan", "Clarke-1880", "MEAN FOR Ethiopia; Sudan", {-166, -15, 204}},
		{"Adindan", "Clarke-1880", "Senegal", {-128, -18, 224}},
		{"Adindan", "Clarke-1880", "Sudan", {-161, -14, 205}},
		{"Afgooye", "Krassovsky", "Somalia", {-43, -163, 45}},
		{"Ain el Abd 1970", "International-1924", "Bahrain", {-150, -250, -1}},
		{"Ain el Abd 1970", "International-1924", "Saudi Arabia", {-143, -236, 7}},
		{"American Samoa 1962", "Clarke-1866", "American Samoa Islands", {-115, 118, 426}},
		{"Anna 1 Astro 1965", "Australian", "Cocos Islands", {-491, -22, 435}},
		{"Antigua Island Astro 1943", "Clarke-1880", "Antigua (Leeward Islands)", {-270, 13, 62}},
		{"Arc 1950", "Clarke-1880", "Botswana", {-138, -105, -289}},
		{"Arc 1950", "Clarke-1880", "Burundi", {-153, -5, -292}},
		{"Arc 1950", "Clarke-1880", "Lesotho", {-125, -108, -295}},
		{"Arc 1950", "Clarke-1880", "Malawi", {-161, -73, -317}},
		{"Arc 1950", "Clarke-1880", "MEAN FOR Botswana; Lesotho; Malawi; Swaziland; Zaire; Zambia; Zimbabwe", {-143, -90, -294}},
		{"Arc 1950", "Clarke-1880", "Swaziland", {-134, -105, -295}},
		{"Arc 1950", "Clarke-1880", "Zaire", {-169, -19, -278}},
		{"Arc 1950", "Clarke-1880", "Zambia", {-147, -74, -283}},
		{"Arc 1950", "Clarke-1880", "Zimbabwe", {-142, -96, -293}},
		{"Arc 1960", "Clarke-1880", "MEAN FOR Kenya; Tanzania", {-160, -6, -302}},
		{"Arc 1960", "Clarke-1880", "Kenya", {-157, -2, -299}},
		{"Arc 1960", "Clarke-1880", "Taanzania", {-175, -23, -303}},
		{"Ascension Island 1958", "International-1924", "Ascension Island", {-205, 107, 53}},
		{"Astro Beacon E 1945", "International-1924", "Iwo Jima", {145, 75, -272}},
		{"Astro DOS 71/4", "International-1924", "St Helena Island", {-320, 550, -494}},
		{"Astro Tern Island (FRIG) 1961", "International-1924", "Tern Island", {114, -116, -333}},
		{"Astronomical Station 1952", "International-1924", "Marcus Island", {124, -234, -25}},
		{"Australian Geodetic 1966", "Australian", "Australia; Tasmania", {-133, -48, 148}},
		{"Australian Geodetic 1984", "Australian", "Australia; Tasmania", {-134, -48, 149}},
		{"Ayabelle Lighthouse", "Clarke-1880", "Djibouti", {-79, -129, 145}},
		{"Bellevue (IGN)", "International-1924", "Efate & Erromango Islands", {-127, -769, 472}},
		{"Bermuda 1957", "Clarke-1866", "Bermuda", {-73, 213, 296}},
		{"Bissau", "International-1924", "Guinea-Bissau", {-173, 253, 27}},
		{"Bogota Observatory", "International-1924", "Colombia", {307, 304, -318}},
		{"Bukit Rimpah", "Bessel", "Indonesia (Bangka & Belitung Ids)", {-384, 664, -48}},
		{"Camp Area Astro", "International-1924", "Antarctica (McMurdo Camp Area)", {-104, -129, 239}},
		{"Campo Inchauspe", "International-1924", "Argentina", {-148, 136, 90}},
		{"Canton Astro 1966", "International-1924", "Phoenix Islands", {298, -304, -375}},
		{"Cape", "Clarke-1880", "South Africa", {-136, -108, -292}},
		{"Cape Canaveral", "Clarke-1866", "Bahamas; Florida", {-2, 151, 181}},
		{"Carthage", "Clarke-1880", "Tunisia", {-263, 6, 431}},
		{"Chatham Island Astro 1971", "International-1924", "New Zealand (Chatham Island)", {175, -38, 113}},
		{"Chua Astro", "International-1924", "Paraguay", {-134, 229, -29}},
		{"Corrego Alegre", "International-1924", "Brazil", {-206, 172, -6}},
		{"Dabola", "Clarke-1880", "Guinea", {-83, 37, 124}},
		{"Datum 73", "International-1924", "Portugal", {-223.237, 110.193, 36.649}},
		{"Datum Lisboa", "International-1924", "Portugal", {-304.046, -60.576, 103.640}},
		{"Deception Island", "Clarke-1880", "Deception Island; Antarctia", {260, 12, -147}},
		{"Djakarta (Batavia)", "Bessel", "Indonesia (Sumatra)", {-377, 681, -50}},
		{"DOS 1968", "International-1924", "New Georgia Islands (Gizo Island)", {230, -199, -752}},
		{"Easter Island 1967", "International-1924", "Easter Island", {211, 147, 111}},
		{"European 1950", "International-1924", "Cyprus", {-104, -101, -140}},
		{"European 1950", "International-1924", "Egypt", {-130, -117, -151}},
		{"European 1950", "International-1924", "England; Channel Islands; Scotland; Shetland Islands", {-86, -96, -120}},
		{"European 1950", "International-1924", "England; Ireland; Scotland; Shetland Islands", {-86, -96, -120}},
		{"European 1950", "International-1924", "Finland; Norway", {-87, -95, -120}},
		{"European 1950", "International-1924", "Greece", {-84, -95, -130}},
		{"European 1950", "International-1924", "Iran", {-117, -132, -164}},
		{"European 1950", "International-1924", "Italy (Sardinia)", {-97, -103, -120}},
		{"European 1950", "International-1924", "Italy (Sicily)", {-97, -88, -135}},
		{"European 1950", "International-1924", "Malta", {-107, -88, -149}},
		{"European 1950", "International-1924", "MEAN FOR Austria; Belgium; Denmark; Finland;France; W Germany; Gibraltar; Greece; Italy; Luxembourg; Netherlands;Norway; Portugal; Spain; Sweden; Switzerland", {-87, -98, -121}},
		{"European 1950", "International-1924", "MEAN FOR Austria; Denmark; France; W Germany; Netherlands; Switzerland", {-87, -96, -120}},
		{"European 1950", "International-1924", "MEAN FOR Iraq; Israel; Jordan; Lebanon; Kuwait; Saudi Arabia; Syria", {-103, -106, -141}},
		{"European 1950", "International-1924", "Portugal; Spain", {-86.277, -108.879, -120.181}},
		{"European 1950", "International-1924", "Tunisia", {-112, -77, -145}},
		{"European 1979", "International-1924", "MEAN FOR Austria; Finland; Netherlands; Norway; Spain; Sweden; Switzerland", {-86, -98, -119}},
		{"Fort Thomas 1955", "Clarke-1880", "Nevis; St. Kitts (Leeward Islands)", {-7, 215, 225}},
		{"Gan 1970", "International-1924", "Republic of Maldives", {-133, -321, 50}},
		{"Geodetic Datum 1949", "International-1924", "New Zealand", {84, -22, 209}},
		{"Graciosa Base SW 1948", "International-1924", "Azores (Faial; Graciosa; Pico; Sao Jorge; Terceira)", {-111.789, 154.869, -44.765}},
		{"Guam 1963", "Clarke-1866", "Guam", {-100, -248, 259}},
		{"Gunung Segara", "Bessel", "Indonesia (Kalimantan)", {-403, 684, 41}},
		{"GUX 1 Astro", "International-1924", "Guadalcanal Island", {252, -209, -751}},
		{"Herat North", "International-1924", "Afghanistan", {-333, -222, 114}},
		{"Hermannskogel Datum", "Bessel (Namibia)", "Croatia -Serbia, Bosnia-Herzegovina", {653, -212, 449}},
		{"Hjorsey 1955", "International-1924", "Iceland", {-73, 46, -86}},
		{"Hong Kong 1963", "International-1924", "Hong Kong", {-156, -271, -189}},
		{"Hu-Tzu-Shan", "International-1924", "Taiwan", {-637, -549, -203}},
		{"Indian", "Everest-1830", "Bangladesh", {282, 726, 254}},
		{"Indian", "Everest-1830-Kalianpur", "India; Nepal", {295, 736, 257}},
		{"Indian", "Everest-1830-Pakistan", "Pakistan", {283, 682, 231}},
		{"Indian 1954", "Everest-1830", "Thailand", {217, 823, 299}},
		{"Indian 1960", "Everest-1830", "Vietnam (Con Son Island)", {182, 915, 344}},
		{"Indian 1960", "Everest-1830", "Vietnam (Near 16N)", {198, 881, 317}},
		{"Indian 1975", "Everest-1830", "Thailand", {210, 814, 289}},
		{"Indonesian", "Indonesian", "Indonesia", {-24, -15, 5}},
		{"Ireland 1965", "Modified-Airy", "Ireland", {506, -122, 611}},
		{"ISTS 061 Astro 1968", "International-1924", "South Georgia Islands", {-794, 119, -298}},
		{"ISTS 073 Astro 1969", "International-1924", "Diego Garcia", {208, -435, -229}},
		{"Johnston Island 1961", "International-1924", "Johnston Island", {189, -79, -202}},
		{"Kandawala", "Everest-1830", "Sri Lanka", {-97, 787, 86}},
		{"Kerguelen Island 1949", "International-1924", "Kerguelen Island", {145, -187, 103}},
		{"Kertau 1948", "Everest-1830-Kertau", "West Malaysia & Singapore", {-11, 851, 5}},
		{"Kusaie Astro 1951", "International-1924", "Caroline Islands", {647, 1777, -1124}},
		{"Korean Geodetic System", "GRS-80", "South Korea", {0, 0, 0}},
		{"L. C. 5 Astro 1961", "Clarke-1866", "Cayman Brac Island", {42, 124, 147}},
		{"Leigon", "Clarke-1880", "Ghana", {-130, 29, 364}},
		{"Liberia 1964", "Clarke-1880", "Liberia", {-90, 40, 88}},
		{"Luzon", "Clarke-1866", "Philippines (Excluding Mindanao)", {-133, -77, -51}},
		{"Luzon", "Clarke-1866", "Philippines (Mindanao)", {-133, -79, -72}},
		{"M'Poraloko", "Clarke-1880", "Gabon", {-74, -130, 42}},
		{"Macau", "International-1924", "Macau", {-203.792, -303.371, -154.020}},
		{"Mahe 1971", "Clarke-1880", "Mahe Island", {41, -220, -134}},
		{"Massawa", "Bessel", "Ethiopia (Eritrea)", {639, 405, 60}},
		{"Merchich", "Clarke-1880", "Morocco", {31, 146, 47}},
		{"Midway Astro 1961", "International-1924", "Midway Islands", {912, -58, 1227}},
		{"Minna", "Clarke-1880", "Cameroon", {-81, -84, 115}},
		{"Minna", "Clarke-1880", "Nigeria", {-92, -93, 122}},
		{"Montserrat Island Astro 1958", "Clarke-1880", "Montserrat (Leeward Islands)", {174, 359, 365}},
		{"Nahrwan", "Clarke-1880", "Oman (Masirah Island)", {-247, -148, 369}},
		{"Nahrwan", "Clarke-1880", "Saudi Arabia", {-243, -192, 477}},
		{"Nahrwan", "Clarke-1880", "United Arab Emirates", {-249, -156, 381}},
		{"Naparima BWI", "International-1924", "Trinidad & Tobago", {-10, 375, 165}},
		{"North American 1927", "Clarke-1866", "Alaska (Excluding Aleutian Ids)", {-5, 135, 172}},
		{"North American 1927", "Clarke-1866", "Alaska (Aleutian Ids East of 180W)", {-2, 152, 149}},
		{"North American 1927", "Clarke-1866", "Alaska (Aleutian Ids West of 180W)", {2, 204, 105}},
		{"North American 1927", "Clarke-1866", "Bahamas (Except San Salvador Id)", {-4, 154, 178}},
		{"North American 1927", "Clarke-1866", "Bahamas (San Salvador Island)", {1, 140, 165}},
		{"North American 1927", "Clarke-1866", "Canada (Alberta; British Columbia)", {-7, 162, 188}},
		{"North American 1927", "Clarke-1866", "Canada (Manitoba; Ontario)", {-9, 157, 184}},
		{"North American 1927", "Clarke-1866", "Canada (New Brunswick; Newfoundland; Nova Scotia; Quebec)", {-22, 160, 190}},
		{"North American 1927", "Clarke-1866", "Canada (Northwest Territories; Saskatchewan)", {4, 159, 188}},
		{"North American 1927", "Clarke-1866", "Canada (Yukon)", {-7, 139, 181}},
		{"North American 1927", "Clarke-1866", "Canal Zone", {0, 125, 201}},
		{"North American 1927", "Clarke-1866", "Cuba", {-9, 152, 178}},
		{"North American 1927", "Clarke-1866", "Greenland (Hayes Peninsula)", {11, 114, 195}},
		{"North American 1927", "Clarke-1866", "MEAN FOR Antigua; Barbados; Barbuda; Caicos Islands; Cuba; Dominican Republic; Grand Cayman; Jamaica; Turks Islands", {-3, 142, 183}},
		{"North American 1927", "Clarke-1866", "MEAN FOR Belize; Costa Rica; El Salvador; Guatemala; Honduras; Nicaragua", {0, 125, 194}},
		{"North American 1927", "Clarke-1866", "MEAN FOR Canada", {-10, 158, 187}},
		{"North American 1927", "Clarke-1866", "MEAN FOR CONUS", {-8, 160, 176}},
		{"North American 1927", "Clarke-1866", "MEAN FOR CONUS (East of Mississippi; River Including Louisiana; Missouri; Minnesota)", {-9, 161, 179}},
		{"North American 1927", "Clarke-1866", "MEAN FOR CONUS (West of Mississippi; River Excluding Louisiana; Minnesota; Missouri)", {-8, 159, 175}},
		{"North American 1927", "Clarke-1866", "Mexico", {-12, 130, 190}},
		{"North American 1983", "GRS-80", "Alaska (Excluding Aleutian Ids)", {0, 0, 0}},
		{"North American 1983", "GRS-80", "Aleutian Ids", {-2, 0, 4}},
		{"North American 1983", "GRS-80", "Canada", {0, 0, 0}},
		{"North American 1983", "GRS-80", "CONUS", {0, 0, 0}},
		{"North American 1983", "GRS-80", "Hawaii", {1, 1, -1}},
		{"North American 1983", "GRS-80", "Mexico; Central America", {0, 0, 0}},
		{"North Sahara 1959", "Clarke-1880", "Algeria", {-186, -93, 310}},
		{"Observatorio Meteorologico 1939", "International-1924", "Azores (Corvo & Flores Islands)", {-425, -169, 81}},
		{"Old Egyptian 1907", "Helmert-1906", "Egypt", {-130, 110, -13}},
		{"Old Hawaiian", "Clarke-1866", "Hawaii", {89, -279, -183}},
		{"Old Hawaiian", "Clarke-1866", "Kauai", {45, -290, -172}},
		{"Old Hawaiian", "Clarke-1866", "Maui", {65, -290, -190}},
		{"Old Hawaiian", "Clarke-1866", "MEAN FOR Hawaii; Kauai; Maui; Oahu", {61, -285, -181}},
		{"Old Hawaiian", "Clarke-1866", "Oahu", {58, -283, -182}},
		{"Oman", "Clarke-1880", "Oman", {-346, -1, 224}},
		{"Ordnance Survey Great Britain 1936", "Airy", "England", {371, -112, 434}},
		{"Ordnance Survey Great Britain 1936", "Airy", "England; Isle of Man; Wales", {371, -111, 434}},
		{"Ordnance Survey Great Britain 1936", "Airy", "MEAN FOR England; Isle of Man; Scotland; Shetland Islands; Wales", {375, -111, 431}},
		{"Ordnance Survey Great Britain 1936", "Airy", "Scotland; Shetland Islands", {384, -111, 425}},
		{"Ordnance Survey Great Britain 1936", "Airy", "Wales", {370, -108, 434}},
		{"Pico de las Nieves", "International-1924", "Canary Islands", {-307, -92, 127}},
		{"Pitcairn Astro 1967", "International-1924", "Pitcairn Island", {185, 165, 42}},
		{"Point 58", "Clarke-1880", "MEAN FOR Burkina Faso & Niger", {-106, -129, 165}},
		{"Pointe Noire 1948", "Clarke-1880", "Congo", {-148, 51, -291}},
		{"Porto Santo 1936", "International-1924", "Porto Santo; Madeira Islands", {-503.362, -247.604, 312.651}},
		{"Provisional South American 1956", "International-1924", "Bolivia", {-270, 188, -388}},
		{"Provisional South American 1956", "International-1924", "Chile (Northern; Near 19S)", {-270, 183, -390}},
		{"Provisional South American 1956", "International-1924", "Chile (Southern; Near 43S)", {-305, 243, -442}},
		{"Provisional South American 1956", "International-1924", "Colombia", {-282, 169, -371}},
		{"Provisional South American 1956", "International-1924", "Ecuador", {-278, 171, -367}},
		{"Provisional South American 1956", "International-1924", "Guyana", {-298, 159, -369}},
		{"Provisional South American 1956", "International-1924", "MEAN FOR Bolivia; Chile; Colombia; Ecuador; Guyana; Peru; Venezuela", {-288, 175, -376}},
		{"Provisional South American 1956", "International-1924", "Peru", {-279, 175, -379}},
		{"Provisional South American 1956", "International-1924", "Venezuela", {-295, 173, -371}},
		{"Provisional South Chilean 1963", "International-1924", "Chile (Near 53S) (Hito XVIII)", {16, 196, 93}},
		{"Puerto Rico", "Clarke-1866", "Puerto Rico; Virgin Islands", {11, 72, -101}},
		{"Pulkovo 1942", "Krassovsky", "Russia", {28, -130, -95}},
		{"Qatar National", "International-1924", "Qatar", {-128, -283, 22}},
		{"Qornoq", "International-1924", "Greenland (South)", {164, 138, -189}},
		{"Reunion", "International-1924", "Mascarene Islands", {94, -948, -1262}},
		{"Rome 1940", "International-1924", "Italy (Sardinia)", {-225, -65, 9}},
		{"S-42 (Pulkovo 1942)", "Krassovsky", "Hungary", {28, -121, -77}},
		{"S-42 (Pulkovo 1942)", "Krassovsky", "Poland", {23, -124, -82}},
		{"S-42 (Pulkovo 1942)", "Krassovsky", "Czechoslavakia", {26, -121, -78}},
		{"S-42 (Pulkovo 1942)", "Krassovsky", "Latvia", {24, -124, -82}},
		{"S-42 (Pulkovo 1942)", "Krassovsky", "Kazakhstan", {15, -130, -84}},
		{"S-42 (Pulkovo 1942)", "Krassovsky", "Albania", {24, -130, -92}},
		{"S-42 (Pulkovo 1942)", "Krassovsky", "Romania", {28, -121, -77}},
		{"S-JTSK", "Bessel", "Czechoslavakia (Prior 1 JAN 1993)", {589, 76, 480}},
		{"Santo (DOS) 1965", "International-1924", "Espirito Santo Island", {170, 42, 84}},
		{"Sao Braz", "International-1924", "Azores (Sao Miguel; Santa Maria Ids)", {-203, 141, 53}},
		{"Sao Tome", "International-1924", "Sao Tome Island", {-176, -650, -82}},
		{"Sapper Hill 1943", "International-1924", "East Falkland Island", {-355, 21, 72}},
		{"Schwarzeck", "Bessel (Namibia)", "Namibia", {616, 97, -251}},
		{"Selvagem Grande 1938", "International-1924", "Salvage Islands", {-289, -124, 60}},
		{"Sierra Leone 1960", "Clarke-1880", "Sierra Leone", {-88, 4, 101}},
		{"South-American", "South-American", "Argentina", {-62, -1, -37}},
		{"South-American", "South-American", "Bolivia", {-61, 2, -48}},
		{"South-American", "South-American", "Brazil", {-60, -2, -41}},
		{"South-American", "South-American", "Chile", {-75, -1, -44}},
		{"South-American", "South-American", "Colombia", {-44, 6, -36}},
		{"South-American", "South-American", "Ecuador", {-48, 3, -44}},
		{"South-American", "South-American", "Ecuador (Baltra; Galapagos)", {-47, 26, -42}},
		{"South-American", "South-American", "Guyana", {-53, 3, -47}},
		{"South-American", "South-American", "MEAN FOR Argentina; Bolivia; Brazil; Chile; Colombia; Ecuador; Guyana; Paraguay; Peru; Trinidad & Tobago; Venezuela", {-57, 1, -41}},
		{"South-American", "South-American", "Paraguay", {-61, 2, -33}},
		{"South-American", "South-American", "Peru", {-58, 0, -44}},
		{"South-American", "South-American", "Trinidad & Tobago", {-45, 12, -33}},
		{"South-American", "South-American", "Venezuela", {-45, 8, -33}},
		{"South Asia", "Modified-Fischer-1960", "Singapore", {7, -10, -26}},
		{"Tananarive Observatory 1925", "International-1924", "Madagascar", {-189, -242, -91}},
		{"Timbalai 1948", "Everest-1830-Timbalai", "Brunei; E. Malaysia (Sabah Sarawak)", {-679, 669, -48}},
		{"Tokyo", "Bessel", "Japan", {-148, 507, 685}},
		{"Tokyo", "Bessel", "MEAN FOR Japan; South Korea; Okinawa", {-148, 507, 685}},
		{"Tokyo", "Bessel", "Okinawa", {-158, 507, 676}},
		{"Tokyo", "Bessel", "South Korea", {-147, 506, 687}},
		{"Tristan Astro 1968", "International-1924", "Tristan da Cunha", {-632, 438, -609}},
		{"Viti Levu 1916", "Clarke-1880", "Fiji (Viti Levu Island)", {51, 391, -36}},
		{"Voirol 1960", "Clarke-1880", "Algeria", {-123, -206, 219}},
		{"Wake Island Astro 1952", "International-1924", "Wake Atoll", {276, -57, 149}},
		{"Wake-Eniwetok 1960", "Hough", "Marshall Islands", {102, 52, -38}},
		{"WGS 1972", "WGS-72", "Global Definition", {0, 0, 0}},
		{"WGS 1984", "WGS-84", "Global Definition", {0, 0, 0}},
		{"Yacare", "International-1924", "Uruguay", {-155, 171, 37}},
		{"Zanderij", "International-1924", "Suriname", {-265, 120, -358}}
