"/ISO-8859-2_Encoding [\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/space		/exclam		/quotedbl	/numbersign	/dollar		/percent	/ampersand	/quotesingle\n\
/parenleft	/parenright	/asterisk	/plus		/comma		/hyphen		/period		/slash\n\
/zero		/one		/two		/three		/four		/five		/six		/seven\n\
/eight		/nine		/colon		/semicolon	/less		/equal		/greater	/question\n\
/at		/A		/B		/C		/D		/E		/F		/G\n\
/H		/I		/J		/K		/L		/M		/N		/O\n\
/P		/Q		/R		/S		/T		/U		/V		/W\n\
/X		/Y		/Z		/bracketleft	/backslash	/bracketright	/asciicircum	/underscore\n\
/grave		/a		/b		/c		/d		/e		/f		/g\n\
/h		/i		/j		/k		/l		/m		/n		/o\n\
/p		/q		/r		/s		/t		/u		/v		/w\n\
/x		/y		/z		/braceleft	/bar		/braceright	/asciitilde	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/uni00A0	/Aogonek	/breve		/Lslash		/currency	/Lcaron		/Sacute		/section\n\
/dieresis	/Scaron		/Scedilla	/Tcaron		/Zacute		/uni00AD	/Zcaron		/Zdotaccent\n\
/degree		/aogonek	/ogonek		/lslash		/acute		/lcaron		/sacute		/caron\n\
/cedilla	/scaron		/scedilla	/tcaron		/zacute		/hungarumlaut	/zcaron		/zdotaccent\n\
/Racute		/Aacute		/Acircumflex	/Abreve		/Adieresis	/Lacute		/Cacute		/Ccedilla\n\
/Ccaron		/Eacute		/Eogonek	/Edieresis	/Ecaron		/Iacute		/Icircumflex	/Dcaron\n\
/Dcroat		/Nacute		/Ncaron		/Oacute		/Ocircumflex	/Ohungarumlaut	/Odieresis	/multiply\n\
/Rcaron		/Uring		/Uacute		/Uhungarumlaut	/Udieresis	/Yacute		/Tcommaaccent	/germandbls\n\
/racute		/aacute		/acircumflex	/abreve		/adieresis	/lacute		/cacute		/ccedilla\n\
/ccaron		/eacute		/eogonek	/edieresis	/ecaron		/iacute		/icircumflex	/dcaron\n\
/dcroat		/nacute		/ncaron		/oacute		/ocircumflex	/ohungarumlaut	/odieresis	/divide\n\
/rcaron		/uring		/uacute		/uhungarumlaut	/udieresis	/yacute		/tcommaaccent	/dotaccent\n\
] def\n",
